function [] = estimate_spectrum(cutoff_energies, elements)
    try
        path_xraydb = evalin('caller','path_xraydb');
    catch
        [file_xraydb, path_xraydb] = uigetfile('xraydb.mat', 'Show me X-ray database file');
        path_xraydb = sprintf([path_xraydb file_xraydb]);
        assignin('base','path_xraydb', path_xraydb);
    end
    
    [edges, lines] = extract_structure(path_xraydb);
    
    try
        exist(cutoff_energies);
    catch
        try
            cutoff_energies = evalin('caller','cutoff_energies');
        catch
            energy1 = input('enter first cutoff energy (in keV)\n','s');
            energy2 = input('enter second cutoff energy (in keV)\n','s');
            energy1 = str2double(energy1);
            energy2 = str2double(energy2);
            cutoff_energies = [min([energy1 energy2]) max([energy1 energy2])];
            assignin('base','cutoff_energies', cutoff_energies);
        end
    end
    
    try
        exist(elements);
    catch
        try
            elements = evalin('caller','elements');
        catch
            answer = 'start';
            counter = 1;
            elements = {};
            while length(answer>0)
                message = sprintf('Please enter element %d\n', counter);
                answer = input(message,'s');
                elements{counter} = answer;
                counter = counter + 1;
            end
            elements(end) = [];
            assignin('base','elements', elements);
        end
    end
    
    
    min_energy = min(cutoff_energies);
    max_energy = max(cutoff_energies);
    
    indices = [edges.energy] - min_energy >= 0;
    edges = edges(indices);
    indices = [edges.energy] - max_energy <= 0;
    edges = edges(indices);
    
    indices = [lines.energy] - min_energy >= 0;
    lines = lines(indices);
    indices = [lines.energy] - max_energy <= 0;
    lines = lines(indices);
    
    reduced_edges = [];
    for i=1:length(elements)
        indices = [edges.element] == elements(i);
        temp_edges = edges(indices);
        reduced_edges = [reduced_edges; temp_edges];
    end
    edges = reduced_edges;
    
    reduced_edges = [];
    indices = [edges.yield] > 0.03;
    reduced_edges = [reduced_edges; edges(indices)];
    edges = reduced_edges;
    
    
    reduced_lines = [];
    prev_element = edges(1).element;
    indices = [lines.element] == edges(1).element;
    temp_lines = lines(indices);
    if ~isempty(temp_lines)
        sub_indices = [temp_lines.initial_level] == edges(1).edge;
        reduced_lines = [reduced_lines; temp_lines(sub_indices)];
    end
    
    for i=2:length(edges)
        if edges(i).element == prev_element
           if ~isempty(temp_lines)
                sub_indices = [temp_lines.initial_level] == edges(i).edge;
                reduced_lines = [reduced_lines; temp_lines(sub_indices)];
           end
        elseif edges(i).element ~= prev_element
           prev_element = string(edges(i).element);
           indices = [lines.element] == edges(i).element;
           temp_lines = lines(indices);
           if ~isempty(temp_lines)
                sub_indices = [temp_lines.initial_level] == edges(i).edge;
                reduced_lines = [reduced_lines; temp_lines(sub_indices)];
           end
        end
    end
    lines = reduced_lines;
    
    reduced_lines = [];
    indices = [lines.intensity] > 0.03;
    reduced_lines = [reduced_lines; lines(indices)];
    
    try
        fls = evalin('caller','fls');
    catch
        fls = [];
    end
    fls.xraydb.reduced_edges = edges;
    fls.xraydb.reduced_lines = reduced_lines;
    assignin('base','fls', fls);
    
    plot_lines(fls)
    
    function [edges, lines] = extract_structure(path_xraydb)
        load(path_xraydb);
        edges = table2struct(edges);
        lines = table2struct(lines);
    end

    function [] = plot_lines(fls)
        label_shift = 0.02;
        for i=1:length(fls.xraydb.reduced_edges)
            element = fls.xraydb.reduced_edges(i).element;
            edge = fls.xraydb.reduced_edges(i).edge;
            energy = fls.xraydb.reduced_edges(i).energy;
            yield = fls.xraydb.reduced_edges(i).yield;
            label = sprintf('%s %s', element, edge);
            stem(energy, yield, 'LineStyle', ':',...
                                'LineWidth', 2,...
                                'Color','red',...
                                'MarkerFaceColor','red',...
                                'MarkerSize', 8, ...
                                'Marker','d')
            hold on
            text(energy, yield+label_shift, label);
        end
        for i=1:length(fls.xraydb.reduced_lines)
            element = fls.xraydb.reduced_lines(i).element;
            line = fls.xraydb.reduced_lines(i).line;
            energy = fls.xraydb.reduced_lines(i).energy;
            intensity = fls.xraydb.reduced_lines(i).intensity;
            label = sprintf('%s %s', element, line);
            stem(energy, intensity, 'LineStyle', ':',...
                                'LineWidth', 2,...
                                'Color','blue',...
                                'MarkerFaceColor','blue',...
                                'MarkerSize', 8, ...
                                'Marker','o')
            hold on
            text(energy, intensity + label_shift, label);
        end
        ylim([0 max([[fls.xraydb.reduced_edges.yield] ...
             [fls.xraydb.reduced_lines.intensity]])+label_shift*2]);
        label = sprintf(['Absorption and emission spectrum for \n' ...
          '%2.2f to %2.2f keV range'], cutoff_energies(1), cutoff_energies(2));
        title(label)
        xlabel('Energy, [keV]')
        ylabel('Intensity/yield, [a.u.]')
        hold off
    end
end