function [fls] = load_fluo(fls)
%% load light variables
scan_num = fls.exp.scan_num;
path_data = fls.paths.data;
path_positions = fls.paths.positions;

%% 
try
    scans_old = evalin('caller','scans_old');
catch
    scans_old = [];
end

loaddata = 1;
if exist('scans_old','var')
    if (numel(scans_old)==numel(scan_num))
        if all(scans_old == scan_num)
            loaddata = 0;
        end
    end
end
%%

num_of_scans = length(scan_num);

if loaddata
    parfor ii = 1:num_of_scans
        filename{ii} = fullfile(sprintf(path_data, scan_num(ii)));

%% check whether the file was already created
        while ~isfile(filename{ii})
            fprintf('could not find scan number %d. will try again now\n', scan_num(ii));
            pause(0.3);
        end
        
        file_size{ii} = dir(filename{ii});
        while file_size{ii}.bytes == 0
            fprintf('scan number %d is loaded but appears empty. will try again now\n', scan_num(ii));
            pause(0.3);
            file_size{ii} = dir(filename{ii});
        end

%% %% %% %% %% %% %% 

        data_str{ii} = io.image_read(filename{ii});
        data{ii} = data_str{ii}.data;
        
        posdata{ii} = beamline.read_omny_pos(sprintf(path_positions, scan_num(ii)));
        x{ii} = -posdata{ii}.Average_x_st_fzp;
        y{ii} = -posdata{ii}.Average_y_st_fzp;
    end


data = cell2mat(data);
x = vertcat(x{:});
y = vertcat(y{:});

Nx = round(sqrt( (max(x)-min(x))*numel(x)/(max(y)-min(y))));
Ny = round(numel(x)/Nx);

xg = linspace(min(x), max(x), Nx);
yg = linspace(min(y), max(y), Ny);
[Xg, Yg] = meshgrid(xg, yg);

num_of_energies = length(data(:, 1));
intermid_data = zeros(num_of_energies, Ny, Nx);

disp('Converting spectral scans to images')
parfor ii = 1:num_of_energies
    F = scatteredInterpolant(x, y, data(ii, :).');
    intermid_data(ii, :, :) = F(Xg, Yg);
end
disp('DONE')

fls.exp.data = intermid_data;

fls.exp.pixel_size_x = abs(xg(2) - xg(1));
fls.exp.pixel_size_y = abs(yg(2) - yg(1));

else
    display('Using data already loaded')
end

scans_old = scan_num;
assignin('base','scans_old', scans_old);
clearvars data_str data intermid_data
end
