%%% A template to plot fluorescence data %%%
%%% mcs_mesh.m could be upgraded to do this job, but here is a script in
%%% the meantime
%%% Feb 24, 2016

%%% In order to use datread(filename) we need in the header at least this
% # channels 8192
% # dim1 4
% # dim2 1
% # number-of-exposures 1
% # end-of-header
% 
% Would also be nice to see the MCS data and match the format approximately 

% return
%%
% clear all
% close all
% flag_low_res = 1;

%%
import beamline.read_omny_pos
import io.*
import 
addpath ../ptycho
addpath ../tomo
addpath ../

scan_array =  scanstomo; %[47:80]; %[4838];%[463:524]; %532:538
home_dir = '/sls/X12SA/Data20/e16588';
output_dir = '/mnt/das-gpfs/work/p16588';
imwrite_folder = sprintf('%s/analysis/offline/fluorescence/',output_dir);

makemovie = 0;
pausetime = 0.2;
movie_filename = sprintf('%s/fluo_projections.avi',output_folder);


%%% Parameters of scan and detector %%%
num_channels = 1024;
% num_channels = 8192;
% For plotting in 2D, set = 1 to ignore %
numlines = 1;
%%% Parameters %%%
omnyposfile = [home_dir '/specES1/scan_positions/scan_%05d.dat']; % Set to empty to ignore and do a mesh, e.g. 
scanning_points = [];   % Example = '+b'
upsampling = 1;         % Ratio of scanning vs plotting points in one dimensions. If (= 1) data points and meshpoints are approx equal


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Choose energies to plot in keV %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%energy_to_plot = 6.49;  % Mn Kb1 edge
%energy_label = 'Mn Kb1';

%energy_to_plot = 6.915;  % Co Ka2 edge
%energy_label = 'Co Ka2';

% energy_to_plot = 8.26;  %
% energy_label = 'Ni Kb1';

% energy_to_plot = 7.4;  %
% energy_label = 'scan';

energy_to_plot = 6.92;  %
energy_label = 'scan';

energy_peak_width = 0.5;      % Specify the FWHM of the integration range

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Get spectrum of a ROI %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%roi_spectrum{1} = [7 8 -7 -6];
%roi_spectrum{2} = [9 10 -7 -6];
%roi_spectrum{3} = [10 11 -7 -6];
%roi_spectrum{1} = []; %[xi xf yi yf] Define the ROI, set empty = [] to ignore
%roi_spectrum{2} = []; %[xi xf yi yf] Define the ROI
% roi_spectrum{3} = [23 23.5 -4.5 -4]; %[xi xf yi yf] Define the ROI

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Calibration of spectrum %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
spectrum_step = 8.905*(8192/num_channels)/5656;
energy_axis = [0:spectrum_step:(num_channels-1)*spectrum_step]; % Empty to see the pixels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

index_center = find(abs(energy_axis-energy_to_plot)==min(abs(energy_axis-energy_to_plot)),1)-1;
index_start  = find(abs(energy_axis-energy_to_plot+energy_peak_width/2)==min(abs(energy_axis-energy_to_plot+energy_peak_width/2)),1)-1;
index_end    = find(abs(energy_axis-energy_to_plot-energy_peak_width/2)==min(abs(energy_axis-energy_to_plot-energy_peak_width/2)),1)-1;
spectrum_range  = [index_start:index_end]; 

% 5109 -> Cu k-alpha  8.047 (by eye)
% 5656 -> Cu k-beta   8.905

loaddata = 1;
if exist('scans_old','var')
    if (numel(scans_old)==numel(scan))
        if all(scans_old == scan)
            loaddata = 0;
        end
    end
end

%
numscans = numel(scan_array);

for idx = 1:numscans
    
    scan = scan_array(idx);

    filename{idx} = fullfile(sprintf('%s/x123/gccDppConsoleLinux/data/x123_%05d.dat',home_dir,scan));
    data_str{idx} = image_read(filename{idx});
    %data = [data data_str{ii}.data];
    data = data_str{idx}.data;
    %data = data(:,1:size(data,2)-1);

    title_str = sprintf('Scan: S%05d - S%05d',scan(1),scan(end));

    if numlines>1
        if mod(numels,numlines) ~= 0
            error('The number of lines is not consistent with the number of elements in the measurement')
        end
        if isempty(spectrum_range)
            img_sp = sum(data,1);
        else
            img_sp = sum(data(spectrum_range,:),1);
        end
        img_sp = reshape(img_sp,[numlines numels/numlines]);
        f3 = figure(3)
        imagesc(img_sp)
        colormap bone
        colorbar
        axis xy equal tight
        title(['Scanning image: ' title_str],'interpreter','none')
    end

    %%%%%%%%%%%%%%%%%%%%%
    %%% Plot spectrum %%%
    %%%%%%%%%%%%%%%%%%%%%
    if 1
        sumdata = sum(data,2);

        f1 = figure(1);
        clf
        if ~isempty(energy_axis)
            semilogy(energy_axis,sum(data,2))
            xlabel('keV')
        end
        title(['Total spectrum ' title_str])
        legendarray = [];
        legendtext = {};
        labelcolortext = [];

        hold on
        if ~isempty(spectrum_range)
            h1 = semilogy([min(spectrum_range) min(spectrum_range)]*spectrum_step,[1 max(sum(data,2))],'k');
            semilogy([max(spectrum_range) max(spectrum_range)]*spectrum_step,[1 max(sum(data,2))],'k');
            legendarray = h1;
            legendtext{end+1} = 'Image';
        end

        hold off
        legend(legendarray,legendtext)

        % figure(2);
        % semilogy(sumdata)
        % title(['Sum of all spectra ' title_str],'interpreter','none')
        % % legend
    end

    %%%%%%%%%%%%%%%%%%%%%
    %%% Interpolate positions %%%
    %%%%%%%%%%%%%%%%%%%%%
    if ~isempty(omnyposfile)
        x = [];
        y = [];
        %for ii = 1:numscans
        ii=1;
            posdata{ii} = read_omny_pos(sprintf(omnyposfile,scan));
            x = [x; -posdata{ii}.Average_x_st_fzp];
            y = [y; -posdata{ii}.Average_y_st_fzp];
        %end

        if isempty(spectrum_range)
            img_sp = sum(data,1).';
        else
            img_sp = sum(data(spectrum_range,:),1).';
        end

        %%% Interpolate %%%
        if flag_low_res 
            Nx = upsampling*round(sqrt( (max(x)-min(x))*numel(x)/(max(y)-min(y))));
            Ny = upsampling^2*round(numel(x)/Nx);
        else
            Nx = round((max(x)-min(x))/p.dx_spec(1)*1e-6);
            Ny = round((max(y)-min(y))/p.dx_spec(2)*1e-6);
        end
        
        F = scatteredInterpolant(x,y,img_sp);
        xg = linspace(min(x),max(x),Nx);
        yg = linspace(min(y),max(y),Ny);
        [Xg,Yg] = meshgrid(xg,yg);
        img_sp_interp = F(Xg,Yg);
        if idx==1
            nx = size(img_sp_interp,2);
            ny = size(img_sp_interp,1);
        else
            object = img_sp_interp;
            % ---- Trim object size
            if size(object,2) > nx       
                object = object(:,1:nx);       
            elseif size(object,2) < nx
                object = padarray(object,[0 nx-size(object,2)],'post');
            end
            if size(object,1) > ny
                if 0 %auto_alignment||get_auto_calibration
                    object = object(1:ny,:);
                else
                    shifty = floor((size(object,1)-ny)/2);
                    object = object([1:ny]+shifty,:);
                end
            elseif size(object,1) < ny
                if 0 %auto_alignment||get_auto_calibration
                    object = padarray(object,[ny-size(object,1) 0],'post');
                else
                    shifty = (ny-size(object,1))/2;
                    object = padarray(object,[ny-size(object,1)-floor(shifty) 0],'post');
                    object = padarray(object,[floor(shifty) 0],'pre');
                end
            end
            img_sp_interp = object;
        end
        
%         min(x)
%         max(x)
%         min(y)
%         max(y)    
%         

        %% Plot
        if 1            
            home_dir = '/sls/X12SA/Data20/e16588';
            if mod(idx,2)==0
                ptycho = load(sprintf('/sls/X12SA/Data20/e16588/analysis/S%05d/wrap_S%05d_S%05d_400x400_run_1_c_unwrapped.mat',scans(idx),scans(idx),scans(idx)+1));
            else
                ptycho = load(sprintf('/sls/X12SA/Data20/e16588/analysis/S%05d/wrap_S%05d_S%05d_400x400_run_1_c_unwrapped.mat',scans(idx),scans(idx)-1,scans(idx)));
            end

            
            figure(3); clf
            
            subplot(2,3,1);
            imagesc(angle(ptycho.object(asize/2:end-asize/2,asize/2:end-asize/2)));
            colorbar
            axis xy equal tight
            
            subplot(2,3,2);
            imagesc(img_sp_interp);
            colorbar
            axis xy equal tight
            title(['Image ' title_str],'interpreter','none')
            if ~isempty(scanning_points)
                hold on
                plot(x,y,scanning_points)
                hold off
            end
            xlabel([energy_label ' ' num2str(energy_to_plot) ' keV'])
                       
            set(gcf,'position',[200 200 1600 900])
            colormap bone
            drawnow
        end

    end

    stack_object(:,:,idx) =  img_sp_interp;

end

scans_old = scan;


%%
f5 = figure(5);
if makemovie
    writeobj = VideoWriter(movie_filename); 
    writeobj.Quality=50; writeobj.FrameRate=5; open(writeobj); 
end
for idx = 1:size(stack_object,3) 
    
    imagesc(stack_object(:,:,idx));
    colormap bone
    colorbar
    axis xy equal tight
    title(sprintf('S%05d (projection %d)',scanstomo(idx), idx),'interpreter','none')
   
    if makemovie
        currFrame = getframe(f5);
        writeVideo(writeobj,currFrame);
    end
    pause(pausetime)
end


if makemovie == 1 
    close(writeobj); 
end

%*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing language this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS software package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.    
