% Deconvolution with specified scanning positions
% 2017-03

% First run: fluorescence_save_projections.m

close all
addpath utils
import fluo.*
import math.*
import plotting.hline
import plotting.vline

%%
scan = 68; %68;
analysis_folder = '/mnt/das-gpfs/work/p16588/analysis/'; %'~/Data10/analysis/';
filenamewithpath = [analysis_folder sprintf('S%05d/object_fluo_%.3f.mat', scan,6.915)];  
x = load(filenamewithpath)

ptycho = load(sprintf('/sls/X12SA/Data20/e16588/analysis/S%05d/wrap_S%05d_S%05d_400x400_run_1_c.mat',scan, scan-1, scan));
edge = ptycho.p.asize(1)/2;

%%
downsample = 16;

N_zpad = 50;
temp = double(x.object(1:downsample:end,1:downsample:end,1));
[m n] = size(temp);
object = zeros(m+N_zpad*2,n+N_zpad*2);
object(N_zpad+1:N_zpad+m,N_zpad+1:N_zpad+n) = temp;

probe = conv2(abs(x.p.probe).^2, ones(downsample,downsample), 'same');
probe = probe(1:downsample:end,1:downsample:end,1);

[W,H] = size(object);

% --- create scanning positions 
%pos = make_circular_scan([20,W-20, 20, H-20], step );
pos_x = round(x.p.positions(x.p.numpts(1)+1:end,2)/downsample);
pos_y = round(x.p.positions(x.p.numpts(1)+1:end,1)/downsample);
pos = [pos_x, pos_y];
pos = pos+N_zpad;
Npos = length(pos);

% --- probe intensity (make probe of given radius in pixels)
%probe = get_kernel(4);
if mod(length(probe),2) == 0
    [m n] = size(probe);
    probe = probe(1:m-1,1:n-1);
end

% --- get the system matrix of deconvolution 

Tblur = geom_mat_blur([W,H], probe);
% show the blurred image 
% img_new = reshape(Tblur*img_0(:), size(img_0));

% Tblur_small = geom_mat_blur(size(img_0),  get_kernel(2));
Tgeom = geom_mat_simpler([W,H], pos, Npos);

%%%%%%%%%%%% SOLVER %%%%%%%%%%%%%%%%%%%%%%%
% true system matrix 
T = Tgeom*Tblur; 

% show the system matrix 
if 0 
    figure(50); clf 
    for i = 1:10:Npos
        imagesc(reshape(full(T(i,:)), [W,H]));
        axis tight equal xy;
        colormap bone; pause(0.1); 
    end
    hold on; plot(pos(:,1),pos(:,2),'ro')
end

% --- create the "measured" data 
% data = T*img_0(:);
data = x.img_sp; %T*object(:);
% add noise 
% data= data + randn(size(data)) * sigma*max(data) ;


% regularization constant 
lambdas = logspace(-4,1, 20);
chi2 = zeros(size(lambdas));
% precalculate products of the system matrices 
TT = T'*T;
Tf = (T'*data(:));

% tichonov solver (regularized LSQ)
N = W*H;
% prepare boundary conditions 
Sv = ones(W,H);
Sh = ones(W,H);
Sh([end],:) = 0;
Sv(:,[end]) = 0;
Sv = Sv(:); Sh = Sh(:);
% get matrices of derivatives (penalize roughness)
By = spdiags([- [~Sv(W+1:end); ones(W,1)], ones(N,1), -[ones(W,1);Sv(1:end-W)] ],[-W, 0,W],N,N);
Bx = spdiags([- [~Sv(2:end); 1], ones(N,1), -[1;Sv(1:end-1)] ],[-1, 0,1],N,N);

% constant that is enforcing sparsity in the freq. space 
alpha = 50;
beta = 1e-3;  % another regularization constant (Avoid division by 0 and positivity constraint)
Niter = 3; % number of inner iteration of the tichonov 

%% Method 1
sigma =1;
if 0
    for i = 1:length( lambdas )
        i
        lam = lambdas(i);
        % initial guess 
        g = ones(N,1);
        for j = 1:Niter
            BB = Bx'*Bx + By'*By + alpha * spdiags(1./(beta*max(g(:))+g(:)),0,N,N);
            A = TT + lam * BB ; % speye(N);
            g = A  \ Tf;
            g = max(0,g); % nonnegativity 
        end
        chi2(i)  = mean(( T*g - data(:) ).^2 ./  sigma.^2 ); 
        figure(1);
        imagesc(reshape(g, W,H)); axis equal tight xy; 
        drawnow
        pause(0.01)
    end
end

%% Method 2
% just a stupid guess of the optimal lambda 
chi2_search = exp(0.1*(max(log(chi2))- min(log(chi2)))+min(log(chi2)));
min_chi2_lam = lambdas(argmin(chi2));

% best_lam = lambdas( argmin(abs(chi2 - chi2_search)+ max(chi2)*(lambdas < min_chi2_lam ) ));
best_lam = lambdas(argmax(chi2 ./ lambdas));

%% Method 3 - random guess
best_lam = 0.02e-1; % larger smooth
alpha = 64; % smaller smooth

g = ones(W,H);

tic
for j = 1:Niter
    BB = Bx'*Bx + By'*By + alpha * spdiags(1./(beta*max(g(:))+g(:)),0,N,N);
    A = TT + best_lam * BB ; % speye(N);
    g = A  \ Tf;
    g = max(0,g); % nonnegativity 
end
f = T*g;
toc

% SHOW FINAL RESULT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
g_re = reshape(g, W,H);
[X,Y] = meshgrid(1:W, 1:H);
roi_plot{1} = [N_zpad:size(object,1)-N_zpad+1];
roi_plot{2} = [N_zpad:size(object,2)-N_zpad+1];

figure(100); clf
subplot(3,1,1);
imagesc((angle(ptycho.object(edge:end-edge+1,edge:end-edge+1)))); axis equal tight xy; title('Ptychography'); colorbar
subplot(3,1,2)
imagesc((object(roi_plot{:}))); axis equal tight xy; title('Original'); colorbar
hold on; plot(pos(:,1)-N_zpad, pos(:,2)-N_zpad, 'w.')

subplot(3,1,3)
imagesc((abs(g_re(roi_plot{:})))); axis equal tight xy; 
title(sprintf('Reconstruction \n alpha %01.f; lam %.3e',alpha, best_lam)); colorbar
hold on; plot(pos(:,1)-N_zpad, pos(:,2)-N_zpad, 'w.')

set(gcf,'color','w');


%%
% subplot(2,3,3)
% imagesc(griddata(pos(:,1), pos(:,2), data,X,Y)); axis off image ;title('Data')
% hold on; plot(pos(:,1)-N_zpad, pos(:,2)-N_zpad, 'k.')
% 
% subplot(2,3,4)
% imagesc(griddata(pos(:,1), pos(:,2), f,X,Y)); axis off image; title('Retrofit')
% subplot(2,3,5)
% MAX = max(data(:));
% imagesc(griddata(pos(:,1), pos(:,2), data - f,X,Y),[-MAX, MAX]); axis off image;title('Difference')
% subplot(2,3,6)
% imagesc(griddata(pos(:,1), pos(:,2), Tgeom*Tblur*object(:),X,Y)); axis off image;title('Ideal reconstruction')


figure(2)
loglog(lambdas, chi2,  '.-'); 
title('Chi2 vs Lambda')
hold on 
hline(1)
vline(best_lam)
hold off
% 

%*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing language this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS software package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.