import fluo.fluorescence_data
import fluo.repeat
import io.spec_read

%% Input
base_path='/sls/X12SA/Data20/e16588/';
output_path = '/mnt/das-gpfs/work/p16588/';
addpath([base_path 'cxs_software/base']);
addpath([base_path 'cxs_software/ptycho/']);
addpath([base_path 'cxs_software/tomo/']);
%addpath ~/Data10/cxs_software/tomo/utils/

%matlabpool open
flag_plot = 0; % Set to 0 because maybe not always working, need to test

%%[*] File management
%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Edit this section %%%
%%%%%%%%%%%%%%%%%%%%%%%%%
scans_all = [47:200]; %[47:438]; %[2978:2979];%[569:1400];%[1414:2008]% [2035:2636]; 
sample_name='wrap_';                 %% Related to reconstruction name
filesuffix = '_400x400_run_1'; % No need to add _c or _recons, it will look for it
use_OMNY_file_angles = 1;        %% Read angles from special file in OMNY, otherwise it uses SPEC dat file
OMNY_angle_file = [base_path 'specES1/dat-files/tomography_scannumbers.txt'];  % Filename with angles
checkangles = false;             %% Compares spec angles with expected
sort_by_angle = 0;            %% If true everything is sorted by angle rather than by the scan numbers
auto_alignment = 0;     %% Uses interferometer positions to prealign projections - for now it will read them from the spec file. 
            % This feature needs a new calibration of the sphere and is not fully tested.
get_auto_calibration = 0;  % If you want to use this recons to get a calibration file for autotomo            
%%% Optional parameters if checkangles = true %%%
projections =1200;
subtomograms = 2;
subs_to_do = 2;                 % spec read is faster with a matlabpool
savedata = 1;                   % 1 to save data to disk.
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%

stack_object = [];
stack_unwrap = [];
deltaslice = [];
deltastack = [];
whichempty = [];
whichexist = [];

ind_old = 1;
ind = [];

auto_tomo_fn_out = ['auto_tomo_' sample_name];


%% Angles
stack_unwrap = [];
deltaslice = [];
deltastack = [];
whichempty = [];
whichexist = [];

tomo_tic = tic;
scans = scans_all;

%%% Check the first reconstruction
analysis_folder = [sprintf('S%5.5d',scans(1))];
analysis_path = [base_path 'analysis/',analysis_folder,'/'];

file = find_ptycho_filename(analysis_path,scans(1),sample_name,filesuffix);
if isempty(file)
    error(['Out of luck - Reconstruction ' file ' not found']);
end

%%%  Read first projection to check size and reconstruction parameters
display(['Reading file: ' file])
load(file);
asize = p.asize;
dims_ob=size(object);
nx=dims_ob(2);
ny=dims_ob(1);

lambda=p.lambda;                         % wavelength [m]
pixsize=p.dx_spec(1);       % reconstructed pixel size [m]
if p.dx_spec(1)~=p.dx_spec(2)
    warning('Pixel size not symmetric - This code cannot handle')
end
factor=lambda/(2*pi*pixsize); 

%%% Creating save folder
first_scan = scans(1);
scans_string=sprintf('S%05d_to_S%05d',scans(1),scans(end));
output_folder=[output_path 'analysis_tomo/tomo_' scans_string filesuffix '/'];

%%%  Check angles %%%
if checkangles
    [scans_check, angles_check] = tomo_angles(projections, subtomograms, ...
        first_scan, subs_to_do); % ignores the repeated 180deg scan.
end
angles = scans*0;

if ~use_OMNY_file_angles
    S = spec_read(base_path,'ScanNr',scans);
    for ii = 1:max(size(scans));
        angles(ii)=S{ii}.samroy;
    end
else
    [S errflag] = read_OMNY_angles(OMNY_angle_file,scans);
    if errflag
        display(['Not all scans found in ' OMNY_angle_file])
        display(['I will remove the angles not found and show you some plots anyway'])
        scans = S.scan(:).';
    end
    angles=S.readout_angle(:).';
end

ind_old = ind;
ind = find(angles<179.99); 
theta = angles(ind);    % Angles not repeated in scan
scanstomo = scans(ind);
num_proj=numel(scanstomo);
[thetasort indsortangle] = sort(theta);

if sort_by_angle
    theta = theta(indsortangle);
    scanstomo = scanstomo(indsortangle);
end

if flag_plot
    figure(1);
    if ~ checkangles
    subplot(2,1,1)
    plot(scanstomo,theta,'ob'); grid on;
    legend('Spec angles')
    subplot(2,1,2)
    plot(diff(thetasort))
    title('Angular spacing'); grid on;
else
    subplot(3,1,1)
    plot(scanstomo,theta,'ob')
    legend('Spec angles')
    hold on
    plot(scans_check,angles_check,'+r')
    hold off
    legend('Spec angles','Expected angles')
    subplot(3,1,2)
    plot(scanstomo,theta-angles_check,'o')
    title('Difference')
    subplot(3,1,3)
    plot(diff(thetasort),'o')
    title('Angular spacing')
    end
    set(gcf,'Outerposition',[139 163 946 815])    %[left, bottom, width, height
end


%% Load data
% scanstomo
flag_low_res = 1;
fluo.fluorescence_data
stack_unwrap = stack_object;

%% Display
if 1
   figure(99);
   for ii = 1:length(theta)
       imagesc(stack_unwrap(:,:,ii));
       title(sprintf('%d',ii));
       axis xy equal tight
       drawnow
       pause(0.2)
   end
end

%% Remove bad projections (to be added)

%% Save stack_object
savedata = 0;
if savedata 
    if ~exist('output_folder','dir'); mkdir(output_folder); end
    saveprojfile = sprintf('%sfluo_data_S%05d_S%05d.mat',output_folder,...
        scanstomo(1),scanstomo(end));
    if exist(saveprojfile,'file')
        display(['File ' saveprojfile ' exists,' ])
        userans = input(['Do you want to overwrite (y/n)? '],'s');
        if strcmp(lower(userans),'y')
            display(['Saving to  ' saveprojfile]);
            save(saveprojfile,'stack_object','scanstomo', 'theta','output_folder','num_proj','-v7.3');
        else
            display(['Did not save ' saveprojfile])
        end
    else
        display(['Saving to  ' saveprojfile]);
        save(saveprojfile,'stack_object','scanstomo', 'theta','output_folder','num_proj','-v7.3');
    end
end


%% Make tomo
filter_type = 'Hann';   % Filter to use for FBP
freq_scale = 1;       % Frequency cutoff
N_slice = size(stack_unwrap,1);

flag_plot_slice = 1;
clear tomogram

%%%%%%%%%%%%%%%%%%%%%%%%%
for slice_num = 1:N_slice
    clear sino
    clear tomo_slice
    sino = squeeze(stack_unwrap(slice_num,:,:));

    %sino = sino(:,[1:23 24:33]); % Remove bad projections
    theta_use = theta; %([1:23 24:33]);

    tomo_slice = iradonfast_v2(sino, theta_use,'linear', filter_type,size(sino,1), freq_scale);   % Calculate slice

    if flag_plot_slice
        [theta_sort idx] = sort(theta_use);
        sino_sort = sino(:,idx);

        figure(1); clf
        subplot(2,2,1);
        plot(theta_sort); hold on; grid on; axis tight
        ylabel('degree');

        subplot(2,2,2);
        imagesc(sino_sort); 
        colormap bone; 
        colorbar;
        axis xy
        title(['Sinogram slice ' num2str(slice_num)])
        set(gcf,'Outerposition',[1 607 600 600]);

        subplot(2,2,3)
        imagesc(tomo_slice); 
        st_title = sprintf('Tomogram slice %d', slice_num);
        title(st_title,'interpreter','none'); colormap bone(256); 
        axis equal tight xy; 
        colorbar; 
        drawnow
    end

    tomogram(:,:,slice_num) = tomo_slice;
end

size(tomogram)
return

%% Display reconstructed volume by slices
%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Edit this section %%%
%%%%%%%%%%%%%%%%%%%%%%%%%
displayaxis = 1;            % 1 coronal, 2 sagital, 3 axial
displayslice = [1:size(tomogram,1)];         % Many slices supported e.g [100:200]. If left empty the center slice is shown, 

makemovie = 0;
pause_time = 0.1;
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%
tic
if ~exist('output_folder','dir'); mkdir(output_folder); end 

if makemovie    % Open movie file
    output_name = sprintf('S%05d_S%05d',scanstomo(1),scanstomo(end));
    movie_filename = [output_folder output_name sprintf('_fluo_axis%d_fine_2.avi',displayaxis)];
    display(['Saving movie to  ' movie_filename]);
    writeobj = VideoWriter(movie_filename); writeobj.Quality=50;writeobj.FrameRate=5; open(writeobj); 
end

for showslice = displayslice
    % Determine sagital, coronal or axial slices
    switch displayaxis
    case 1
        sliceview = squeeze(tomogram(showslice,:,:))';
        sectionstring = 'Coronal';
    case 2
        sliceview = squeeze(tomogram(:,showslice,:))';
        sectionstring = 'Sagital';
    case 3
        sliceview = squeeze(tomogram(:,:,showslice));
        sectionstring = 'Axial';
    otherwise
        error('Unexpected choice of displayaxis')
    end
    
    f1000=figure(1000);
    imagesc(sliceview); colorbar; 
    axis tight xy equal; colormap hot
    caxis([min(tomogram(:)) max(tomogram(:))])
    %caxis([0 max(tomogram(:))])
    title(sprintf('%s slice %d',sectionstring,showslice));
    drawnow
    if makemovie
        currFrame = getframe(f1000);
        writeVideo(writeobj,currFrame);
    end
    
    pause(pause_time)
end

if makemovie == 1 
    close(writeobj); 
end
toc

%% Save Tiff files for 3D visualization with external program
%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Edit this section %%%
%%%%%%%%%%%%%%%%%%%%%%%%%
tiff_subfolder_name = [];      % Name for subfolder, default if left empty
save_as_stack = false;      % = false will write one file per slice
tiff_compression = 'none';   % = 'none' or = 'packbits'  lossless compression  Not much benefit has been observed
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%

if isempty(tiff_subfolder_name)
    tiff_subfolder_name=['TIFF_delta_' filter_type '_freqscl_' sprintf('%0.2f',freq_scale) '_fluo'];
end
tiff_folder_name = [output_folder tiff_subfolder_name];
if ~exist(tiff_folder_name)
    mkdir(tiff_folder_name); 
else
    display(['Folder exists: ' tiff_folder_name])
    userans = input(['Do you want to overwrite TIFFs in this folder (y/n)? '],'s');
    if strcmp(lower(userans),'y')
        display(['Overwritting']);
    else
        display('Writting of TIFFs aborted')
        return
    end
end

low_cutoff=min(tomogram(:));
high_cutoff=max(tomogram(:));

for j=1:size(tomogram,3);
    img16bit=tomogram(:,:,j)-low_cutoff;
    img16bit= img16bit/(high_cutoff-low_cutoff);
    img16bit=img16bit*(2^16-1);
    if save_as_stack
        image_filename_with_path = [tiff_folder_name '/tomo_delta_' scans_string '_' ...
            filter_type '_freqscl_' sprintf('%0.2f',freq_scale) '.tif'];
        if j == 1
            imwrite(uint16(img16bit),image_filename_with_path,'tiff',...
                'Compression',tiff_compression);
        else
            imwrite(uint16(img16bit),image_filename_with_path,'tiff',...
                'Compression',tiff_compression,'WriteMode','append');
        end
    else
        imwrite(uint16(img16bit),[tiff_folder_name '/tomo_delta_' scans_string '_' ...
            filter_type '_freqscl_' sprintf('%0.2f',freq_scale) '_' sprintf('%04d',j) '.tif'],'tiff','Compression',tiff_compression);
    end
    display(['writing slice ', sprintf('%04d',j), ' of ', sprintf('%03d',size(tomogram,3)), ' in ',tiff_folder_name])
end

fid=fopen([output_folder tiff_subfolder_name '_cutoffs.txt'],'w');
fprintf(fid, '# low_cutoff = %e\n', low_cutoff);
fprintf(fid, '# high_cutoff = %e\n', high_cutoff);
fprintf(fid, '# pixel size = %e\n', pixsize);
fclose(fid);





%*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing language this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS software package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.


