% FUNCTION T = geom_mat_blur(Np, kernel)
    % generates a sparse matrix that emulates convolution with given kernel
    % Example conv2(img,ker,'same') is same as reshape(T*img(:), Npix)

%*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing language this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS software package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.    
    
function T = geom_mat_blur(Np, kernel)
    
    N = prod(Np);

    thresh = 0.001; % minimal kernel value , improve sparsity 

    kernel = kernel./sum(kernel(:));

    % W = spdiags(ones(Npix,1),0, Npix, Npix);
    if any(mod(size(kernel)/2,1)==0)
        error('Kernel size should be odd')
    end
    
    Nk = floor(size(kernel)/2);
    [X,Y] = meshgrid(-Nk(1):Nk(1), -Nk(2):Nk(2));

    Nker = numel(kernel);
    
    array = X*Np(1)+Y;
    array = array(:);
    kernel = single(kernel(:));

    S = zeros(N,Nker, 3, 'single');
    S(:, :, 1) = repmat(1:N, Nker, 1)';
    [X,Y] = meshgrid(1:N, array);
    S(:, :, 2) = single(X)'+single(Y)';
    S(:, :, 3) = repmat(kernel, 1, N)';

    S = reshape(S, [], 3);

    S(S(:,2) < 1 | S(:,2) > N | S(:,3) < thresh,:) = [];
    S = double(S);

    T = sparse(S(:,1),S(:,2), S(:,3), N, N);


end
