    % FUNCTION T =  geom_mat_simpler(Npix, positions, weights, varargin)
    % creates a sparse matrix that will cause shift of the vector into
    % given positions 
    % example: 
    %  img = imread('cameraman')
    %  T*img(:)
    
    %*-----------------------------------------------------------------------*
%|                                                                       |
%|  Except where otherwise noted, this work is licensed under a          |
%|  Creative Commons Attribution-NonCommercial-ShareAlike 4.0            |
%|  International (CC BY-NC-SA 4.0) license.                             |
%|                                                                       |
%|  Copyright (c) 2017 by Paul Scherrer Institute (http://www.psi.ch)    |
%|                                                                       |
%|       Author: CXS group, PSI                                          |
%*-----------------------------------------------------------------------*
% You may use this code with the following provisions:
%
% If the code is fully or partially redistributed, or rewritten in another
%   computing language this notice should be included in the redistribution.
%
% If this code, or subfunctions or parts of it, is used for research in a 
%   publication or if it is fully or partially rewritten for another 
%   computing language the authors and institution should be acknowledged 
%   in written form in the publication: “Data processing was carried out 
%   using the “cSAXS software package” developed by the CXS group,
%   Paul Scherrer Institut, Switzerland.” 
%   Variations on the latter text can be incorporated upon discussion with 
%   the CXS group if needed to more specifically reflect the use of the package 
%   for the published work.
%
% A publication that focuses on describing features, or parameters, that
%    are already existing in the code should be first discussed with the
%    authors.
%   
% This code and subroutines are part of a continuous development, they 
%    are provided “as they are” without guarantees or liability on part
%    of PSI or the authors. It is the user responsibility to ensure its 
%    proper use and the correctness of the results.

function T =  geom_mat_simpler(Npix, positions, weights)
    
    thresh = 1e-3; % lower values are removed to improve sparsity 


    Npos = size(positions,1);
    

    X = floor(positions);
    dX = 1 - (positions - X);
    intens = ones(Npos,1);



    M0 = ones(4,1);
    M1 = [00,0;0,1;1,0;1,1];

    W = zeros(Npos,4);
    W(:,1) = dX(:,1) .* dX(:,2);
    W(:,2) = (1-dX(:,1)) .* dX(:,2);
    W(:,3) = dX(:,1) .* (1-dX(:,2));
    W(:,4) = (1-dX(:,1)) .* (1-dX(:,2));
    W = bsxfun(@times, W, intens);
    W = reshape(W',[],1);

    
    X = replicate(X, 4,1) + repeat(M1,Npos, 1);
    Y = replicate( (1:Npos)', 4,1) ;
    
    ind = (X(:,1)-1)*Npix(1)+X(:,2);
    S(:, 1) = Y;
    S(:, 2) = ind;
    S(:, 3) = W;
    
    S = S( S(:,3) >  thresh, : );

    T = sparse(double(S(:,1)), double(S(:,2)), double(S(:,3)), Npos, prod(Npix) );
    
    
end
