#!/bin/bash
#1                               +––––––––– ––+
#                        hdf5    |            |   hdf5
#                       +––––––->| procPtycho +–––--->                    (multiproc)
#                                |            |
#                                +––––––––––––+
#
#2                               +––––––––––––+             +–––––––––––+
#                        hdf5    |            |    8081     |           |
#                      +––––––-->| procPtycho +–––––––––––+>| zmqWriter | (not multiproc)
#                                |            |             |           |
#                                +––––––––––––+             +–––––––––––+
#
#3 +-----------+                 +––––––––––––+
#  |           |   8080          |            |   hdf5
#  | zmqImgSrc +––––––––––––––-->| procPtycho +–––--->                    (not multiproc)
#  |           |                 |            |
#  +-----------+                 +––––––––––––+
#
#4 +-----------+                 +––––––––––––+             +–––––––––––+
#  |           |   8080          |            |    8081     |           |
#  | zmqImgSrc +––––––––––––––-->| procPtycho +–––––––––––+>| zmqWriter | (not multiproc)
#  |           |                 |            |             |           |
#  +-----------+                 +––––––––––––+             +–––––––––––+
#
source configTest.sh

DSTELEM=entry/data/out
SRCHDF=$INDIR/cSAXS_sxdm_2014_07_omny_commissioning/eiger/S00000-00999/S00033/e15285_1_00033_00000_00000.h5
SRCELEM=eh5/images

RunTest()
{
case "$1" in
1) #1 hdf -> hdf
sed -e s"|\$OUTDIR|$OUTDIR|" -e s"|\$INDIR|$INDIR|" $LXR/test/ptycho1.json > /tmp/ptycho.json
$LXR/procPtycho.py --meta /tmp/ptycho.json --cmpr s-zlib4 --multiproc
#$LXR/procPtycho.py --meta /tmp/ptycho.json --cmpr s-zlib4
#$LXR/procPtycho.py --meta ptycho1.json --cmpr s-zlib4 --multiproc --oldHdfStruct
#$LXR/procPtycho.py --meta ptycho1.json --cmpr s-zlib4 --oldHdfStruct
;;
2) #2 hdf -> zmq
sed -e s"|\$OUTDIR|$OUTDIR|" -e s"|\$INDIR|$INDIR|" $LXR/test/ptycho2.json > /tmp/ptycho.json
$XT0 "$LXR/zmqWriter.py --addr tcp://*:8081 --dst hdf $OUTDIR/procPtycho2.hdf5 $DSTELEM --timeout 5000 -v;$ENDCMD"&
$LXR/procPtycho.py --meta /tmp/ptycho.json --cmpr s-zlib4
;;
3) #3 zmq -> hdf
sed -e s"|\$OUTDIR|$OUTDIR|" -e s"|\$INDIR|$INDIR|" $LXR/test/ptycho3.json > /tmp/ptycho.json
$XT0 "$LXR/zmqImageSource.py -v --addr=tcp://*:8080 --src hdf $SRCHDF $SRCELEM -s;sleep 1"&
$LXR/procPtycho.py --meta /tmp/ptycho.json --cmpr s-zlib4
;;
4) #4 zmq -> zmq
sed -e s"|\$OUTDIR|$OUTDIR|" -e s"|\$INDIR|$INDIR|" $LXR/test/ptycho4.json > /tmp/ptycho.json
$XT0 "$LXR/zmqImageSource.py -v --addr tcp://*:8080 --src hdf $SRCHDF $SRCELEM -s;sleep 1"&
$XT1 "$LXR/zmqWriter.py --addr tcp://*:8081 --dst hdf $OUTDIR/procPtycho4.hdf5 $DSTELEM --timeout 5000 -v;sleep 1"&
$LXR/procPtycho.py --meta /tmp/ptycho.json --cmpr s-zlib4
;;
*) # ------- bug ---------
echo "Unknown parameters"
;;
esac
}

for idx in $*; do
  echo RunTest $idx
  RunTest $idx
  WaitAll
  if [ -z $NO_UI ]; then
    h5pyViewer $OUTDIR/procPtycho$idx.hdf5
  fi
done


#ls -l $OUTDIR/procPtycho*
WaitAll

