#!/bin/bash

#zmqWriterREST.sh 0 0
#zmqWriterREST.sh 0 2
#zmqWriterREST.sh 2 3
#zmqWriterREST.sh 2 2

#  +-----------+
#  |           |   8080
#  | zmqImgSrc |+--------------->
#  |           |
#  +-----------+

#zmqImgSrc sends images via zmq
# 0.  no cmpr  PCO raw images
# 1.  zlib4      PCO raw images
# 2.  cbf files



#          +-----------+
#   8080   |           |
# -------->| zmqWriter |
#          |           |
#          +-----------+
#                ^
#                |
#                |
#                +
#            REST 8090

#zmqWriter writes a hdf file. it is started as a REST server
# 0. hdf5
# 1. hdf5 cmpr:s-zlib4
# 2. hdf5 cmpr:s-zlib4 multiproc
# 3. cbf files (only works if source is cbf files)
source configTest.sh
if [ `hostname` == "pc9477" ]; then
  HOSTSRC=`hostname`
  HOSTDST=`hostname`
  CURLCMD=
else
  HOSTSRC=`hostname`
  HOSTDST=`hostname`
  #HOSTDST=xbl-daq-20
  #CURLCMD=$CURLCMD',"user":"'$USER'"'
fi

function StartSrc
{
case "$1" in
h)
echo 'HHHEEEELLLPPP'
;;
0) # ------- 0 ---------
$XT0 "$LXR/zmqImageSource.py --src raw uint16 2560 2160 $INDIR/PCO/ -v -s -i --addr=tcp://*:8080 -v ;sleep 1d"&
PIDzmqImageSource=$!
;;
1) # ------- 1 ---------
$XT0 "$LXR/zmqImageSource.py --src raw uint16 2560 2160 $INDIR/PCO/ -v -s -i --addr=tcp://*:8080 -v --cmpr=s-zlib4 ;sleep 1d"&
PIDzmqImageSource=$!
;;
2) # ------- 2 ---------
$XT0 "$LXR/zmqImageSource.py -v -i --addr=tcp://*:8080 --src file $INDIR/stxm_pinhole/pilatus_1/S00000-00999/S00150/ $OUTDIR/zmqWrRST_CBF;sleep 1d"&
PIDzmqImageSource=$!
CURLCMD=$CURLCMD',"cbf2idx":[0,0,0,1,0,1]'
;;
t) # ------- TEST ---------
$XT0 "$LXR/zmqImageSource.py --src raw uint16 2560 2160 $INDIR/PCO/ -v -s -i --numFrames 100 --addr=tcp://*:8080 -v ;sleep 1d"&
PIDzmqImageSource=$!
;;
*) # ------- bug ---------
echo "Unknown parameters"
esac
}

function StartWriterREST
{
echo ">>>>>>>>>>>><< parameters"  $@

if [ $HOSTDST == `hostname` ]; then
  $XT1 "$LXR/zmqWriter.py rest --port=8090 -v;sleep 1d"&
  PIDzmqWriter=$!
  echo "PIDzmqWriter $PIDzmqWriter"
  echo "Wait till zmqWriter started..."
  WaitRESTServer http://localhost:8090
fi
echo "send commands to REST server..."
case "$1" in
h)
echo 'HHHEEEELLLPPP'
;;
0) # ------- 0 ---------
set -x;curl -i -X PUT http://$HOSTDST:8090/cam1/run -d '{"addr":"tcp://'$HOSTSRC':8080", "dst":["hdf","'$OUTDIR/zmqWrRST$2.hdf5'","entry/data/dataset"]'$CURLCMD'}';set +x
;;
1) # ------- 1 ---------
set -x;curl -i -X PUT http://$HOSTDST:8090/cam1/run -d '{"addr":"tcp://'$HOSTSRC':8080", "dst":["hdf","'$OUTDIR/zmqWrRST$2.hdf5'","entry/data/dataset"],"cmpr":"s-zlib4"'$CURLCMD'}';set +x
;;
2) # ------- 2 ---------
set -x;curl -i -X PUT http://$HOSTDST:8090/cam1/run -d '{"addr":"tcp://'$HOSTSRC':8080", "dst":["hdf","'$OUTDIR/zmqWrRST$2.hdf5'","entry/data/dataset"],"cmpr":"s-zlib4","multiproc":true,"verbose":511'$CURLCMD'}';set +x
;;
3) # ------- 3 ---------
set -x;curl -i -X PUT http://$HOSTDST:8090/cam1/run -d '{"addr":"tcp://'$HOSTSRC':8080", "dst":["file"], "numFrm":101, "ifType":"PULL"'$CURLCMD'}';set +x
;;
t) # ------- TEST ---------
set -x;curl -i -X PUT http://$HOSTDST:8090/cam1/run -d '{"addr":"tcp://'$HOSTSRC':8080", "dst":["hdf","'$OUTDIR/zmqWrRST$2.hdf5'","entry/data/dataset"],"cmpr":"s-zlib4","numFrm":100,"multiproc":true'$CURLCMD'}';set +x
;;
*) # ------- bug ---------
echo "Unknown parameters"
;;
esac
set +x
}

StartSrc $1
StartWriterREST $2 $1$2

#echo "check page http://$HOSTDST:8090/ in firefox"
#firefox http://$HOSTDST:8090/

echo "wait frame 100 or max 10 sec..."
set -x;curl -i -X PUT http://$HOSTDST:8090/cam1/wait -d '{"timeout":10000, "frmCnt":100}';set +x
echo "stop file writing..."
set -x;curl -i -X PUT http://$HOSTDST:8090/cam1/stop;set +x
#set -x;ls -lh $OUTDIR/zmqWrRST*;set +x

if [ -z $NO_UI ]; then
  WaitAll
else
  kill $PIDzmqImageSource $PIDzmqWriter
fi

