#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
loads a maximum of 20 images into ram
do the image loop over these images to measure the performance of the viewer.
'''

if __name__ == '__main__':
    #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import wx
import os,h5py,time
import zmq
import numpy as np
import utilities as ut
from h5pyViewer.GLCanvasImg import *
from Clamp import ClampIn,ClampOut,ClampLst

class FpsMeter():
  def Reset(self):
    try: del self.ts
    except AttributeError:pass
    try: del self.fps
    except AttributeError:pass
  def Log(self):
    try: tsPrev=self.ts
    except AttributeError:
      self.ts=time.time()
      return ''
    ts=time.time()
    delta=(ts-tsPrev)
    try: fps=self.fps
    except AttributeError:
      fps=1./delta
    else:
      decay=.05
      fps=self.fps*(1.-decay)+1./delta*decay
      print decay

    self.ts=ts
    self.fps=fps
    #return 'time: %7.4f avg fps:%7.2f'%(delta,fps)
    return 'fps:%6.2f'%(fps)

class ImageGLFrame(wx.Frame):
  def __init__(self, parent, title, args):
    self.args=args
    if not title:
      title=self.__class__.__name__# __file__

    style = wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE
    wx.Frame.__init__(self, parent, title=title, size=wx.Size(850, 650), style=style)
    path=__file__
    try:symPath=os.readlink(path) #follow symbolic link
    except (AttributeError,OSError) as e:pass
    else:
      path=symPath
    #path=os.path.abspath(path)
    path=os.path.dirname(path)+'/zmq2imgGL.ico'
    icon = wx.Icon(path, wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)
    canvas=GLCanvasImg(self,self.SetStatusCB)
    self.Centre()
    self.canvas=canvas
    self.BuildMenu()
    self.fpsMeter=FpsMeter()
    self.OnRunStop(None)

  def BuildMenu(self):
    mnBar = wx.MenuBar()
    #-------- Edit Menu --------
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Setup Colormap', 'Setup the color mapping');self.Bind(wx.EVT_MENU, self.canvas.OnColmapSetup, mnItem)
    mnItem=mn.Append(wx.ID_ANY, 'Run Image Loop', 'Run/Stop Image Loop');self.Bind(wx.EVT_MENU, self.OnRunStop, mnItem)
    self.nmConnect=mnItem
    #mnItem=mn.Append(wx.ID_ANY, 'Linear Mapping', 'Use a linear values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLin, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Log Mapping', 'Use a logarithmic values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLog, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Invert X-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    #self.mnIDxAxis=mnItem.GetId()
    #mnItem=mn.Append(wx.ID_ANY, 'Invert Y-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    mnBar.Append(mn, '&Edit')
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Help', 'How to use the image viewer');self.Bind(wx.EVT_MENU, self.canvas.OnHelp, mnItem)
    mnBar.Append(mn, '&Help')

    self.SetMenuBar(mnBar)
    self.CreateStatusBar()

  @staticmethod
  def SetStatusCB(obj,mode,v):
    if mode==0:
      obj.SetStatusText( "Pos:(%d,%d) Value:%d"%v,0)

  def OnRunStop(self,event):
    nmConnect=self.nmConnect
    lbl=nmConnect.GetItemLabel()
    strTbl=('Run', 'Stop')
    if hasattr(self,'imgLst'):
      del self.imgLst
      lbl=lbl.replace(strTbl[1],strTbl[0],1)
      self.Unbind(wx.EVT_IDLE)
    else:
    #def InitImgList(self,num):
      clpIn=ClampIn(self.args.src)
      imgLst=[]
      #if not num:
      #  num=sys.maxint
      num=20
      while len(imgLst)<num:
        arr=clpIn.GetData()
        if arr==None:
          break
        imgLst.append(arr)
      self.imgLst=imgLst
      self.imgLstIdx=0
      lbl=lbl.replace(strTbl[0],strTbl[1],1)
      self.fpsMeter.Reset()
      self.Bind(wx.EVT_IDLE,self.OnIdle)
    nmConnect.SetItemLabel(lbl)
    pass


  def OnIdle(self, event):
    canvas=self.canvas
    #print 'OnIdle',self.imgLstIdx,self.fpsMeter.Log()
    self.SetStatusText('OnIdle %d '%self.imgLstIdx+self.fpsMeter.Log(),0)

    l=len(self.imgLst)
    arr=self.imgLst[self.imgLstIdx%l];self.imgLstIdx+=1
    canvas.data=arr
    canvas.UpdateImg()
    #canvas.OnPaint(None)#force to repaint, Refresh and Update do not force !
    self.canvas.Refresh(False)

if __name__ == '__main__':
  import os,sys,argparse #since python 2.7
  def GetParser(required=True):
    (h,t)=os.path.split(sys.argv[0]);cmd='\n  '+(t if len(h)>3 else sys.argv[0])+' '
    exampleCmd=('--src hdf /home/scratch/detectorData/e14472/scan_00033.hdf5 entry/data/pilatus_1',)
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog='\nExamples:'+''.join(map(lambda s:cmd+s,exampleCmd))+'\n ')
    parser.add_argument('--src',    nargs='+', required=True, help='data source (see below for examples)')
    return parser
    args = parser.parse_args()
    return args

  class App(wx.App):
    def OnInit(self):
      parser=GetParser()
      #parser=GetParser(False) # debug with exampleCmd
      args = parser.parse_args()
      frame = ImageGLFrame(None,None,args)
      frame.Show()
      self.SetTopWindow(frame)
      return True

  app = App()
  app.MainLoop()
