#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
implements an image view to show a colored image of a hdf5 dataset.
'''

if __name__ == '__main__':
    #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import wx
import os,h5py
import zmq
import numpy as np
import utilities as ut
from CbfFile import *
from h5pyViewer.GLCanvasImg import *

class DlgZMQSetup(wx.Dialog):
  def __init__(self,parent):
    wx.Dialog.__init__(self,parent,-1,'ZMQ Setup')
    args=parent.args
    #glImg=parent.glImg
    #glColBar=parent.glColBar
    txtSvr=wx.StaticText(self,-1,'addr')
    txtInterf=wx.StaticText(self,-1,'Interface')
    txtQSz=wx.StaticText(self,-1,'queue size')
    self.edSvr=edSvr=wx.TextCtrl(self,-1,args.addr)
    self.cbInterf=cbInterf=wx.ComboBox(self, -1, choices=ZMQImageGLFrame.ifDict.keys(), style=wx.CB_READONLY)
    self.edQSz=edQSz=wx.TextCtrl(self,-1,'%d'%args.queueSz)

    #key = (key for key,value in ZMQImageGLFrame.ifDict.items() if value==args.ifType).next()
    for i,(k,v) in enumerate(ZMQImageGLFrame.ifDict.iteritems()):
      if v==args.ifType:
        cbInterf.SetSelection(i)
        break

    sizer=wx.BoxSizer(wx.VERTICAL)
    fgs=wx.FlexGridSizer(4,2,5,5)
    fgs.Add(txtSvr,0,wx.ALIGN_RIGHT)
    fgs.Add(edSvr,0,wx.EXPAND)
    fgs.Add(txtInterf,0,wx.ALIGN_RIGHT)
    fgs.Add(cbInterf,0,wx.EXPAND)
    fgs.Add(txtQSz,0,wx.ALIGN_RIGHT)
    fgs.Add(edQSz,0,wx.EXPAND)
    sizer.Add(fgs,0,wx.EXPAND|wx.ALL,5)

    txtSvr.SetFocus()

    btns =  self.CreateButtonSizer(wx.OK|wx.CANCEL)
    sizer.Add(btns,0,wx.EXPAND|wx.ALL,5)
    self.SetSizer(sizer)
    sizer.Fit(self)

class ZMQImageGLFrame(wx.Frame):
  ifDict={'PULL':zmq.PULL,'SUB':zmq.SUB}

  strTbl=('Connect ZMQ', 'Diconnect ZMQ')
  def __init__(self, parent, title, args):
    self.args=args
    if not title:
      title=self.__class__.__name__# __file__

    style = wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE
    wx.Frame.__init__(self, parent, title=title, size=wx.Size(850, 650), style=style)
    path=__file__
    try:symPath=os.readlink(path) #follow symbolic link
    except (AttributeError,OSError) as e:pass
    else:
      path=symPath
    path=os.path.abspath(path)
    #path=os.path.dirname(path)
    icon = wx.Icon(path[:path.rfind('.')]+'.ico', wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)
    canvas=GLCanvasImg(self,self.SetStatusCB)
    self.Centre()
    self.canvas=canvas
    self.BuildMenu()
    if args.addr:
      self.ConnectZMQ()
    else:
      self.svrAdr='tcp://pc8934.psi.ch:8080'

  def BuildMenu(self):
    mnBar = wx.MenuBar()
    #-------- Edit Menu --------
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Setup Colormap', 'Setup the color mapping');self.Bind(wx.EVT_MENU, self.canvas.OnColmapSetup, mnItem)
    mnItem=mn.Append(wx.ID_ANY, ZMQImageGLFrame.strTbl[0], 'Connect/Disconnect to a ZMQ source');self.Bind(wx.EVT_MENU, self.OnConnectZMQ, mnItem)
    self.nmConnect=mnItem
    #mnItem=mn.Append(wx.ID_ANY, 'Linear Mapping', 'Use a linear values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLin, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Log Mapping', 'Use a logarithmic values to color mapping ');self.Bind(wx.EVT_MENU, self.OnMapLog, mnItem)
    #mnItem=mn.Append(wx.ID_ANY, 'Invert X-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    #self.mnIDxAxis=mnItem.GetId()
    #mnItem=mn.Append(wx.ID_ANY, 'Invert Y-Axis', kind=wx.ITEM_CHECK);self.Bind(wx.EVT_MENU, self.OnInvertAxis, mnItem)
    mnBar.Append(mn, '&Edit')
    mn = wx.Menu()
    mnItem=mn.Append(wx.ID_ANY, 'Help', 'How to use the image viewer');self.Bind(wx.EVT_MENU, self.canvas.OnHelp, mnItem)
    mnBar.Append(mn, '&Help')

    self.SetMenuBar(mnBar)
    self.CreateStatusBar()

  @staticmethod
  def SetStatusCB(obj,mode,v):
    if mode==0:
      obj.SetStatusText( "Pos:(%d,%d) Value:%d"%v,0)

  def OnConnectZMQ(self,event):
    if hasattr(self,'sock'):
      self.DisconnectZMQ()
    else:
      dlg=DlgZMQSetup(self)
      if dlg.ShowModal() == wx.ID_OK:
        args=self.args
        args.addr=dlg.edSvr.GetValue()
        args.ifType=ZMQImageGLFrame.ifDict.values()[dlg.cbInterf.GetCurrentSelection()]
        args.edQSz=int(dlg.edQSz.GetValue())
        self.ConnectZMQ()
      dlg.Destroy()
    pass


  def ConnectZMQ(self):
    args=self.args
    nmConnect=self.nmConnect
    self.SetTitle(args.addr)
    self.sock=ut.ZMQ.Socket(args.__dict__,zmq.PULL,4)
    self.Bind(wx.EVT_IDLE, self.OnIdle)
    nmConnect.SetItemLabel(ZMQImageGLFrame.strTbl[1])

  def DisconnectZMQ(self):
    nmConnect=self.nmConnect
    del self.sock
    self.Unbind(wx.EVT_IDLE)
    nmConnect.SetItemLabel(ZMQImageGLFrame.strTbl[0])

  def OnIdle(self, event):
    sock=self.sock
    canvas=self.canvas
    try:
      header = sock.recv_json()
    except zmq.error.Again as e:
      self.SetStatusText('ZMQ Error: '+str(e),0)
      print 'ZMQ Error: '+str(e)
      return
    self.SetStatusText(str(header),0)
    print str(header)

    if sock.getsockopt(zmq.RCVMORE): #@UndefinedVariable
      content= sock.recv()
    if header.has_key('filename'):
      ext=os.path.splitext(header['filename'])[1]
      assert(ext=='.cbf'), 'unsupported file extention'
      cbfParser=CbfParser()
      cbfParser.SetData(content)
      cbfParser.ParseHeader(1)
      (arr,res)=cbfParser.GetBinary()
    else:
      dtype=np.dtype(header['type'])
      shape=tuple(reversed(header['shape']))
      try: cmpr=header['cmpr'] #if the zmq-chunk is compressed, use it
      except KeyError as e:
        arr=np.frombuffer(content ,dtype=dtype)
        arr.shape=shape
      else:
        try: UnCmpr=self.UnCmpr
        except AttributeError as e:
          self.UnCmpr=UnCmpr=ut.GetChunkBuildFunc(cmpr)
        arr=np.ndarray(shape=shape,dtype=dtype)
        arr=UnCmpr.Decompr(np.frombuffer(content ,dtype=np.ubyte),arr)

    canvas.data=arr
    try:
      canvas.UpdateImg()
      canvas.OnPaint(None)#force to repaint, Refresh and Update do not force !
    except:
      print 'zmq2ImgGL:OnIdle() failed'
    self.canvas.Refresh(False)

if __name__ == '__main__':
  import os,sys,argparse #since python 2.7
  def GetParsedArgs(required=True):
    ifType='PULL'
    queueSz=4
    (h,t)=os.path.split(sys.argv[0]);cmd='\n  '+(t if len(h)>3 else sys.argv[0])+' '
    exampleCmd=('--addr=tcp://localhost:8080',
                '--addr=tcp://pc9477.psi.ch:8080',
                '--addr=tcp://*:8080',)

    epilog='''Examples:
  a zmq source is needed e.g.:
   zmqImageSource.py --src rand uint16 2560 2160 6 --addr=tcp://*:8080'''+''.join(map(lambda s:cmd+s,exampleCmd))+'\n '

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog=epilog)
    parser.add_argument('--addr', required=True, help='address of the zmq connection')
    parser.add_argument('--timeout',  type=int,  help='receive timeout in ms')
    parser.add_argument('--if',  dest='ifType', choices=ZMQImageGLFrame.ifDict,default=ifType, help='interface type (default:'+ifType+')')
    parser.add_argument('--qs',  dest='queueSz', type=int,default=queueSz, help='queue size (default:'+str(queueSz)+')')

    args = parser.parse_args()
    args.ifType=ZMQImageGLFrame.ifDict[args.ifType]
    return args

  class App(wx.App):
    def OnInit(self):
      args = GetParsedArgs()
      frame = ZMQImageGLFrame(None,args.addr,args)
      frame.Show()
      self.SetTopWindow(frame)
      return True

  app = App()
  app.MainLoop()
