#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2014 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
converts .hdf5 files from dectris to .cbf files skipping and summing a desired amount of images.
'''

import h5py, zmq, ctypes
from Clamp import ClampLst,ClampIn

import sys,os,time,glob,stat,json,re,zmq
import numpy as np
import traceback
import logging
import h5py
import utilities as ut
from libDetXR import LibDetXR
LibDetXR()
import multiprocessing as mproc

_log=logging.getLogger(__name__)
if __name__ == '__main__':
  logging.basicConfig(level=logging.DEBUG,format='%(levelname)s:%(module)s:%(lineno)d:%(funcName)s:%(message)s ')
  pass

from CbfFile import *

def LogOut(s):
  log=logging.root.handlers[0].stream
  log.write(s)
  log.flush()


class DectrisStream2Cbf():

  def __init__(self,args):
    self.args=args

  def Run(self):
    args=self.args
    verb=args.verbose
    try: os.makedirs(os.path.dirname(args.dst))
    except OSError: pass
    self.sock=ut.ZMQ.Socket(args.src[1],zmq.PULL,4)
    if args.multiproc:
      assert('not yet supported')
      jobs = []
      maxJobs=args.multiproc
      if maxJobs<0:
        maxJobs=mproc.cpu_count()
      assert(maxJobs>0)
      for idx in range(maxJobs):
        p = mproc.Process(target=DectrisStream2Cbf.ExportCBF, args=(self,idx,maxJobs))
        jobs.append((p,None))
        p.start()
      _log.info('waiting child processed...')
      for (p,pipeParent) in jobs:
        #(idx,res)=pipeParent.recv()
        p.join()
      _log.info('finished')
      pass
    else:
      self.ExportCBF(0,1)

  def ExportCBF(self,idxJob,numJobs):
    sock = self.sock
    args=self.args
    numFrm=args.get('numFrm')
    self.cbfWr=CbfWriter()

    frmCnt=0
    while True:
      try:
        try:
          header=ut.ConvUtf8(sock.recv_json())
        except ValueError as e: #simplejson.decoder.JSONDecodeError as e:
          if args.verbose&2:
            LogOut('*')
          elif args.verbose&4:
            _log.warning(str(e))
          continue
        htype=header['htype']
        if htype.startswith('dheader'):
          self.htype_dheader(sock,header)
        elif htype.startswith('dimage'):
          self.htype_dimage(sock,header);frmCnt+=1
        if numFrm and frmCnt>=numFrm:
          break
      except zmq.error.Again as e:
        if args.verbose&8:
          _log.info('zmq timeout after %d ms'%sock.RCVTIMEO)
        elif args.verbose&2:
          LogOut('-')
    return

  def htype_dheader(self,sock,header):
    cbfWr=self.cbfWr
    verbose=self.args.verbose
    while sock.getsockopt(zmq.RCVMORE):
      h=ut.ConvUtf8(sock.recv_json()) #general data

      if verbose&4:
        print h
      try:
        htype=h['htype']
      except KeyError:
        c='''_array_data.header_contents
;
# Detector: {description}, S/N {detector_number}
# {data_collection_date}
# Pixel_size {x_pixel_size} m x {y_pixel_size} m
# Silicon sensor, thickness {sensor_thickness} m
# Exposure_time {count_time} s
# Exposure_period {frame_time} s
# Tau = 199.1e-09 s
# Count_cutoff 1249858 counts
# Threshold_setting: 6200 eV
# Gain_setting: mid gain (vrf = -0.200)
# N_excluded_pixels = 289
# Excluded_pixels: badpix_mask.tif
# Flat_field: FF_p24-0109-F_E12400_T6200_vrf_m0p20.tif
# Trim_file: p24-0109-F_E12400_T6200_vrf_m0p20.bin
# Image_path: /ramdisk/e10003/xtest3/20140506/xtal1/screen/
# Wavelength {wavelength} A
# Detector_distance {detector_distance} m
# Detector_Voffset 0.00000 m
# Beam_xy ({beam_center_x}, {beam_center_y}) pixels
# Flux 31897415409.7
# Filter_transmission 0.1010
# Start_angle 0.0000 deg.
# Angle_increment 0.5000 deg.
# Phi 0.0000 deg.
# Chi 0.0000 deg.
# Oscillation_axis OMEGA
;'''.format(**h)
        cbfWr.header['contents']=c.replace('\n','\r\n')
      else:
        if (htype.startswith('dflatfield') or
            htype.startswith('dpixelmask') or
            htype.startswith('dcountrate_table')):
          content= sock.recv(copy=False)
          if verbose&4:
            print 'content: %d bytes'%len(content)
          pass

  def htype_dimage(self,sock,header):
    cbfWr=self.cbfWr
    verbose=self.args.verbose
    while sock.getsockopt(zmq.RCVMORE):
      h=ut.ConvUtf8(sock.recv_json()) #general data
      if verbose&4:
        print h
      elif verbose&1:
        LogOut('.')
      try:
        htype=h['htype']
      except KeyError:
        pass
      else:
        if htype.startswith('dimage_d'): #dconfig "dimage_d
          content= sock.recv(copy=False)
          if verbose&4:
            print 'content: %d bytes'%len(content)
          #fix header:
          if h['encoding']=='lz4<':
            #DecompressLZ4
            out=np.ndarray(shape=h['shape'][::-1],dtype=np.dtype(h['type']))
            res=LibDetXR.DecompressLZ4(ctypes.c_char_p(content.bytes), len(content), out.ctypes.data, out.nbytes)
            cbfWr.SetData(out)
            fn=self.args.dst%(header['frame']+1)
            cbfWr.Write(fn)

        if htype.startswith('dconfig'):
          pass

if __name__ == '__main__':
  def LogConfig(args):
    logging.basicConfig()
    hdlr=logging.StreamHandler()
    fmt = logging.Formatter('%(levelname)s:%(name)s:%(lineno)d:%(funcName)s:%(message)s')
    #fmt = logging.Formatter('%(levelname)s:%(message)s')
    hdlr.setFormatter(fmt)
    del logging.root.handlers[:]
    logging.root.addHandler(hdlr)
    logging.root.setLevel(logging.DEBUG)
    if args.verbose&1:
      lvl=logging.INFO
    else:
      lvl=logging.WARNING

    logLvls={'CbfFile'    : logging.WARN,
             'DatParser'  : logging.WARN,
             'utilities'  : logging.WARN,
             'libDetXR'   : logging.WARN,
             __name__     : logging.INFO,
            }
    for k,v in logLvls.iteritems():
      logging.getLogger(k).setLevel(v)
    pass

  import argparse #since python 2.7
  def RunWithArgs():

    if len(sys.argv)>1 and sys.argv[1][:4]=='Test':
      print eval(sys.argv[1]+'()')
      return

    (h,t)=os.path.split(sys.argv[0]);cmd='\n  '+(t if len(h)>3 else sys.argv[0])+' '
    exampleCmd=('-v --addr tcp://localhost:8080 --dst /tmp/zmqWriter/DectrisEigerFrame%.5d.cbf',
                )

    epilog='''\n

the verbose bits are:
   bit0(0x01): basic
   bit1(0x02): progress
   bit3(0x04): cbffiles
   bit2(0x08):
   bit4(0x10):
   bit5(0x20):
   bit6(0x40):
   default is 1+2=3 (no -v)
   -v is 1+2+4=7
   -v0 is turn all off\n\n
\n\nExamples:'''+''.join(map(lambda s:cmd+s,exampleCmd))+'\n '

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog=epilog)
    ifType='PULL'
    queueSz=4

    parser.add_argument('--addr',       required=True, help='host:port of the zmq connection')
    parser.add_argument('--if',  dest='ifType', choices=ut.ZMQ.ifDict,default=ifType, help='interface type (default:'+ifType+')')
    parser.add_argument('--qs',  dest='queueSz', type=int,default=queueSz, help='queue size (default:'+str(queueSz)+')')
    parser.add_argument('--timeout',                   type=int,      help='receive timeout in ms')
    parser.add_argument('--dst',        required=True, help='output file pattern')
    parser.add_argument('-v',         dest='verbose',  type=int,  nargs='?', const=15, default=3,  help='verbose level (see below)')
    parser.add_argument('-m', '--multiproc', type=int, nargs='?', const=-1, default=0,help='use multiprocessing method topo,[numproc]')
    parser.add_argument('--numFrames',  dest='numFrm',  type=int, default=long(h5py.h5s.UNLIMITED)>>1, help='maximal number of frames to receive')
    args = parser.parse_args()
    args=ut.dotdict(args.__dict__)
    args.src=['zmq',{'addr':args.addr,'qs':args.queueSz,'if':args.ifType}]
    if args.timeout!=None:
      args.src[1]['timeout']=args.timeout
    for k in ('addr','queueSz','ifType','timeout'):
      del args[k]

    LogConfig(args)
    DectrisStream2Cbf(args).Run()
    pass



#------------------ Main Code ----------------------------------
  #quick in mix arrays '{"sssd":[5,40,10,1]}'
  RunWithArgs()
  #Test1()
