#!/usr/bin/env python
'''
*-----------------------------------------------------------------------*
|                                                                       |
|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
|                                                                       |
|  Author:    Thierry Zamofing (thierry.zamofing@psi.ch)                |
|                                                                       |
|  Purpose:   testing libDetXR functionality.                           |
*-----------------------------------------------------------------------*
'''

#from __future__ import division,print_function
from libDetXR import *
from CbfFile import *
from FileNameGen import *


import traceback
import numpy as np
import matplotlib as mpl
mpl.use('WxAgg')
import utilities as ut

def test1():
  '''get LibDetXR Version'''
  LibDetXR()
  print('GetVersion LibDetXR : ',LibDetXR.GetVersion())

def test2():
  '''compare ByteShuffle with SSE optimized Shuffle32Bit
  -> 8.040785 ByteShuffle 3.085955 Shuffle32Bit '''
  arr=np.array(range(2**24),dtype=np.uint8)
  arr2=np.ndarray(shape=arr.shape,dtype=arr.dtype)
  arr3=np.ndarray(shape=arr.shape,dtype=arr.dtype)
  #arr32=arr.view(np.uint32)
  #res=LibDetXR.ByteShuffle(arr.ctypes.data, arr2.ctypes.data, arr.nbytes, arr.itemsize)
  ut.StopWatch().Start()
  for i in range(1000):
    res=LibDetXR.ByteShuffle(arr.ctypes.data, arr2.ctypes.data, arr.nbytes, 4)
  ut.StopWatch().Log('ByteShuffle')
  for i in range(1000):
    res=LibDetXR.Shuffle32Bit(arr.ctypes.data, arr3.ctypes.data, arr.nbytes)
  ut.StopWatch().Log('Shuffle32Bit')
  print (arr2==arr3).all()

def test3():
  '''test Convert4b8b'''
  num=32*1
  #array could be not 16 byte aligned
  arrRand=np.ndarray(buffer=np.random.bytes(num/2),shape=(num/2,),dtype=np.uint8)
  arr=np.ndarray(shape=(num/2,),dtype=np.uint8)
  arr[:]=arrRand[:]
  arr2=np.ndarray(shape=(num,),dtype=arr.dtype)
  #np.random.bytes(8)
  arr2[:]=255
  res=LibDetXR.Convert4b8b(arr.ctypes.data, arr2.ctypes.data, num)
  print res
  print map(hex,arr)
  print map(hex,arr2)

def test4():

  print 'uint8 CompressZLIBCompressZLIB/DecompressZLIB'
  a=np.ones(512,dtype=np.uint8)
  a[10]=75;a[30]=55
  b=np.zeros(a.shape,dtype=a.dtype)
  c=np.zeros(a.shape,dtype=a.dtype)
  print a.nbytes,b.nbytes
  cmpSz=LibDetXR.CompressZLIB(a.ctypes.data, a.nbytes, b.ctypes.data, b.nbytes,6)
  print b[:cmpSz]
  res=LibDetXR.DecompressZLIB(b.ctypes.data, cmpSz, c.ctypes.data, c.nbytes)
  print c[0:32]

  print '\nuint32 CompressCBF/DecompressCBF'
  a=np.ndarray(buffer=np.random.bytes(32*4),shape=(32), dtype=np.uint32)
  a.flags.writeable=True
  a[:]/=10000
  a[:5]/=10000;a[7:12]/=100;a[12:]/=10000
  b=np.zeros(a.size*a.itemsize,dtype=np.uint8)
  c=np.zeros(a.shape,dtype=a.dtype)
  #a=np.array((24,  6, 131, 2707, 97, 227, 85,  290, 2459, 53, 103, 32,  12, 86, 37, 40),dtype=np.uint32)
  print a
  cmpSz=LibDetXR.CompressCBF(a.ctypes.data, a.nbytes, b.ctypes.data, b.nbytes,a.itemsize)
  print b[:cmpSz],cmpSz
  rawSz=LibDetXR.DecompressCBF(b.ctypes.data, cmpSz, c.ctypes.data, c.nbytes,c.itemsize)
  print c[:rawSz/c.itemsize],rawSz


  print '\nuint16 CompressCBF/DecompressCBF'
  a=a.view(dtype=np.uint16)[0::2].copy()
  b=np.zeros(a.size*a.itemsize,dtype=np.uint8)
  c=np.zeros(a.shape,dtype=a.dtype)
  print a
  cmpSz=LibDetXR.CompressCBF(a.ctypes.data, a.nbytes, b.ctypes.data, b.nbytes,a.itemsize)
  print b[:cmpSz],cmpSz
  rawSz=LibDetXR.DecompressCBF(b.ctypes.data, cmpSz, c.ctypes.data, c.nbytes,c.itemsize)
  print c[:rawSz/c.itemsize],rawSz

  pass


if __name__ == '__main__':

  def _GetchSetup():
    #modified from this site (INTERESTING CODE!)
    #http://stackoverflow.com/questions/1052107/reading-a-single-character-getch-style-in-python-is-not-working-in-unix
    if not globals().has_key('sys'):
      global sys;import sys
    pf=sys.platform
    if pf=="win32":
      if not globals().has_key('msvcrt'):
        global msvcrt;import msvcrt
      return msvcrt.getch
    else:
      for imp in ('tty','termios'):
        if not globals().has_key(imp):
          globals()[imp]=__import__(imp)
      return _GetchUnix
  def _GetchUnix():
    fd = sys.stdin.fileno()
    try:
      old_settings = termios.tcgetattr(fd)
      tty.setraw(sys.stdin.fileno())
      ch = sys.stdin.read(1)
      termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
    except termios.error as e:
      ch = sys.stdin.read(1)
      sys.stdin.read(1)
      etype, value, tb = sys.exc_info()
      filename, lineno, name, line =traceback.extract_tb(tb,1)[0]
      excStr=''.join(traceback.format_exception_only(etype, value))
      sys.stderr.write(excStr[:-1]+': File "%s", line %d, in %s\n' % (filename,lineno,name))
    return ch

  getch = _GetchSetup()
  #For windows only this line is enough:
  #from msvcrt import getch


  def MenuSelection():
    entry=(
      ('1', test1 ),
      ('2', test2 ),
      ('3', test3 ),
      ('4', test4 ),
    )
    while True:
      print '-'*20
      for (key,func) in entry:
        print key,func.func_name+'()',func.__doc__
      print 'x','','exit'
      if type(sys.stdin)==file:
        c=getch() #using from console
        print(c)
      else:
        c=raw_input() #using from debugger
      if c=='x':
        break
      for (key,func) in entry:
        if key==c:
          try:
            func() # run the function
          except BaseException as err:
            #etype, value, tb = sys.exc_info()
            #traceback.print_tb(tb)
            #sys.stderr.write(traceback.format_exception_only(etype, value)[0])
            #sys.stderr.write(repr(value))
            traceback.print_exc()
  #MenuSelection()
  test4()
  pass
