#!/bin/bash

#RESTServer on port 8081 starts following configuration:
#
#0 +-----------------------+
#  |                       |
#  | RESTServerChildSample |
#  |                       |
#  +-----------------------+
#
#1 +-----------+                 +-----------+
#  |           |   8080          |           |
#  | zmqImgSrc |+--------------->| zmqWriter |
#  |           |                 |           |
#  +-----------+                 +-----------+
#

#2 +---src-----+                 +---wrt-----+
#  |           |   8080          |           |
#  | zmqImgSrc |+--------------->| zmqWriter | multiproc
#  |           |                 |           |
#  +-----------+                 +-----------+
#                                  | in clamp
#                                  |                +–––moment-––+
#                                  |  8090          |            |   hdf5
#   this is the same as            |----------––––->| procMoment +–––---> singleproc
#   zmqOutConnect.sh 6             |                |            |
#                                  |                +––––––––––––+
#                                  |                +––-roiStat-–––+
#                                  |  8091          |              |   hdf5
#                                  |----------––––->| procRoitStat +–––---> singleproc
#                                  |                |              |
#                                  |                +––––––––––––––+
#                                  |                +–––stxm–––––+
#                                  |  8092          |            |   hdf5
#                                  |--------––––––->| procSTXM   +–––---> singleproc
#                                  |                |            |
#                                  |                +––––––––––––+
#                                  |                +–––ptycho-––+
#                                  |  8093          |            |   hdf5
#                                  |----------––––->| procPtycho +–––---> singleproc
#                                                   |            |
#                                                   +––––––––––––+

source configTest.sh
rm -rf /tmp/REST*.log

$XT0 "watch -n1 -d ls -l '/tmp/REST*.log'"&
PIDRESTLog=$!

echo "check running RESTServer"
pgrep -lf RESTServer.py
TMP=`pgrep -lf RESTServer.py | wc -l`

if [ $TMP != '0' ]; then
  echo "there are $TMP running RESTServer processes kill them with:"
  echo 'pkill -f RESTServer.py'
fi

echo "starting RESTServer"
$XT1 "$LXR/RESTServer.py --childProcLogFile"&
#$XT1 "$LXR/RESTServer.py"&
PIDRESTServer=$!
WaitRESTServer http://localhost:8081

WaitRESTChild()
{
JOBS=1
echo 'wait all child finished. Printing running childs:'
while [ $JOBS != '0' ]; do
  JOBS=`ls /tmp/REST*.log | wc -l`
  echo -n $JOBS' '
  sleep 5
  curl -i -X GET http://localhost:8081/all
done
}

SendCurl()
{
case "$1" in
0) # ------- 0 ---------
curl -i -X PUT http://localhost:8081/dummy1/run -d '["RESTServerChildSample","zamofing_t",{"abc":19,"someattr":"somevalues"}]'
curl -i -X GET http://localhost:8081/all
sleep 1
curl -i -X PUT http://localhost:8081/dummy1/stop
sleep 1
curl -i -X DELETE http://localhost:8081/dummy1
echo tested > $OUTDIR/RESTServer0.log
sleep 1
;;
1) # ------- 1 ---------
curl -i -X PUT http://localhost:8081/src1/run -d '["zmqImageSource","zamofing_t",{"src":["rand","uint16",2560,2160,100],"verbose":true}]'
curl -i -X PUT http://localhost:8081/wrt1/run -d '["zmqWriter","zamofing_t",{"verbose":19,"addr":"tcp://localhost:8080", "dst":["hdf","'$OUTDIR'/RESTServer1.hdf5","entry/data/dataset"]}]'
curl -i -X GET http://localhost:8081/all
echo "Sleeping 3 seconds..."
sleep 3
curl -i -X PUT http://localhost:8081/src1/stop #source must stop first, because it has no timeout and would block
curl -i -X PUT http://localhost:8081/wrt1/stop
curl -i -X GET http://localhost:8081/all
WaitRESTChild
curl -i -X GET http://localhost:8081/all
curl -i -X DELETE http://localhost:8081/src1
curl -i -X DELETE http://localhost:8081/wrt1
;;
2) # ------- 1 ---------
V=3
#V=63

SRCHDF=$INDIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/scan_00106.hdf5
#SRCHDF=$INDIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/scan_00106-00161.hdf5
SRCELEM=/entry/data/pilatus_1
ANADIR=$INDIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/analysis/data
DSTELEM=entry/data/out
CRLPUT='curl -i -X PUT http://localhost:8081'

$CRLPUT/src/run -d '["zmqImageSource","zamofing_t",{"addr":"tcp://*:8080","src":["hdf","'$SRCHDF'","'$SRCELEM'"],"stopCmd":true}]'
CLAMP='["in,zmq,addr:tcp://localhost:8090,qs:4,if:PUSH","in,zmq,addr:tcp://localhost:8091,qs:4,if:PUSH","in,zmq,addr:tcp://localhost:8092,qs:4,if:PUSH","in,zmq,addr:tcp://localhost:8093,qs:4,if:PUSH"]'
$CRLPUT/wrt/run -d '["zmqWriter","zamofing_t",{"verbose":3,"addr":"tcp://localhost:8080", "dst":["hdf","'$OUTDIR'/RESTSvr20.hdf5","entry/data/dataset"],"multiproc":true,"clamp":'$CLAMP'}]'

#$CRLPUT/moment/run -d '["zmqConsoleSink","zamofing_t",{"addr":"tcp://*:8090", "numFrames":100}]'
$CRLPUT/moment/run -d '["procMoment","zamofing_t",{"src":["zmq","addr:tcp://*:8090","qs:4","if:PULL"],"dst":["hdf","'$OUTDIR'/RESTSvr21.hdf5","'$DSTELEM'"],"matMsk":"'$ANADIR'/pilatus_valid_mask.mat","procmode":"c"}]'

#$CRLPUT/roiStat/run -d '["zmqConsoleSink","zamofing_t",{"addr":"tcp://*:8091","verbose":3}]'
$CRLPUT/roiStat/run -d '["procRoiStat","zamofing_t",{"src":["zmq","addr:tcp://*:8091","qs:4","if:PULL"],"dst":["hdf","'$OUTDIR'/RESTSvr22.hdf5","'$DSTELEM'"],"matRoi":["'$ANADIR'/pilatus_valid_mask.mat","'$ANADIR'/pilatus_integration_mask.mat"]}]'

#$CRLPUT/stxm/run -d '["zmqConsoleSink","zamofing_t",{"addr":"tcp://*:8092","verbose":3}]'
$CRLPUT/stxm/run -d '["procSTXM","zamofing_t",{"src":["zmq","addr:tcp://*:8092","qs:4","if:PULL"],"dst":["hdf","'$OUTDIR'/RESTSvr23.hdf5","'$DSTELEM'"],"roiSize":[0,70]}]'

#$CRLPUT/ptycho/run -d '["zmqConsoleSink","zamofing_t",{"addr":"tcp://*:8093","verbose":3}]'
sed -e s"|\$OUTDIR|$OUTDIR|" -e s"|\$INDIR|$INDIR|" $LXR/test/ptychoRESTSvr.json > /tmp/ptycho.json
$CRLPUT/ptycho/run -d '["procPtycho","zamofing_t",{"meta":"/tmp/ptycho.json","cmpr":"s-zlib4"}]'

curl -i -X GET http://localhost:8081/all

WaitRESTChild
ls -l $OUTDIR/RESTSvr2*
;;
*) # ------- bug ---------
echo "Unknown parameters"
;;
esac
}



echo "check page http://localhost:8081/ in firefox"
#firefox http://localhost:8081/
echo "sending configurations to restServer"



for idx in $*; do
  SendCurl $idx
done
if [ -z $NO_UI ]; then
  read -p "press return to kill all subprocesses"
fi
kill `jobs -p`

#watch -n1 -d ps -o pid,%cpu,time,cmd f --width 120


