#!/bin/bash

#1,2,3,4
#  +-----------+                +-----------+
#  |           |   8080         |           | no compression
#  | zmqImgSrc +--------------->| zmqWriter | compression, singleproc, multiproc
#  |           |                |           |
#  +-----------+                +-+---------+
#  (pilatus-1.0,chunk-1.0)        |(in,raw,cmpr)
#                                 |
#                                 | 8081
#                                 |
#                                 v
#                               +--------------+
#                               |              |
#                               |zmqConsoleSink|
#                               |              |
#                               +--------------+

#5 +-----------+                +-----------+
#  |           |   8080         |           |
#  | zmqImgSrc +--------------->| zmqWriter | compression, multiproc
#  |           |                |           |
#  +-----------+                +-+---------+
#  chunk-1.0                      | in
#                                 |
#                                 | 8081
#                                 |
#                                 v
#                               +----------+
#                               |          |
#                               | zmq2imgGL|
#                               |          |
#                               +----------+


#6 +-----------+                 +-----------+
#  |           |   8080          |           |
#  | zmqImgSrc |+--------------->| zmqWriter | multiproc
#  |           |                 |           |
#  +-----------+                 +-----------+
#                                  | in clamp
#                                  |                +––––––––––––+
#                                  |  8090          |            |   hdf5
#                                  |----------––––->| procMoment +–––---> singleproc
#                                  |                |            |
#                                  |                +––––––––––––+
#                                  |                +––––––––––––––+
#                                  |  8091          |              |   hdf5
#                                  |----------––––->| procRoitStat +–––---> singleproc
#                                  |                |              |
#                                  |                +––––––––––––––+
#                                  |                +––––––––––––+
#                                  |  8092          |            |   hdf5
#                                  |--------––––––->| procSTXM   +–––---> singleproc
#                                  |                |            |
#                                  |                +––––––––––––+
#                                  |                +––––––––– ––+
#                                  |  8093          |            |   hdf5
#                                  |----------––––->| procPtycho +–––---> singleproc
#                                                   |            |
#                                                   +––––––––––––+



#sample of zmqOutConnector
source configTest.sh

RunTest()
{
case "$1" in
0) #0 chunk-1.0,no compression,in
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint32 256 216 100 -v;$ENDCMD"&
$XT1 "$LXR/zmqConsoleSink.py --addr tcp://*:8081 --numFrames 100 -v;$ENDCMD"&
$XT2 "$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/OutCon0.hdf5 entry/data/dataset --numFrames 100 --timeout 1000 -v --clamp in,zmq,addr:tcp://localhost:8081,qs:4,if:PUSH;$ENDCMD"&
;;
1)  #1 chunk-1.0,singleproc s-zlib4,in
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint32 256 216 100 -v;$ENDCMD"&
$XT1 "$LXR/zmqConsoleSink.py --addr tcp://*:8081 --numFrames 100 -v;$ENDCMD"&
$XT2 "$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/OutCon1.hdf5 entry/data/dataset --numFrames 100 --timeout 1000 -v --cmpr s-zlib4 --clamp in,zmq,addr:tcp://localhost:8081,qs:4,if:PUSH;$ENDCMD"&
;;
2) #2 chunk-1.0,multiproc s-zlib4,in
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint32 256 216 100 -v;$ENDCMD"&
$XT1 "$LXR/zmqConsoleSink.py --addr tcp://*:8081 --numFrames 100 -v;$ENDCMD"&
$XT2 "$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/OutCon2.hdf5 entry/data/dataset --numFrames 100 --timeout 1000 -v --cmpr s-zlib4 --multiproc --clamp in,zmq,addr:tcp://localhost:8081,qs:4,if:PUSH;$ENDCMD"&
;;
3) #3 chunk-1.0,singleproc s-zlib4,cmpr
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint32 256 216 100 -v;$ENDCMD"&
$XT1 "$LXR/zmqConsoleSink.py --addr tcp://*:8081 --numFrames 100 -v;$ENDCMD"&
$XT2 "$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/OutCon3.hdf5 entry/data/dataset --numFrames 100 --timeout 1000 -v --cmpr s-zlib4 --clamp cmpr,zmq,addr:tcp://localhost:8081,qs:4,if:PUSH;$ENDCMD"&
;;
4) #4 chunk-1.0,multiproc s-zlib4,cmpr
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src inc uint32 256 216 100 -v;$ENDCMD"&
$XT1 "$LXR/zmqConsoleSink.py --addr tcp://*:8081 --numFrames 100 -v;$ENDCMD"&
$XT2 "$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/OutCon4.hdf5 entry/data/dataset --numFrames 100 --timeout 1000 -v --cmpr s-zlib4 --multiproc --clamp cmpr,zmq,addr:tcp://localhost:8081,qs:4,if:PUSH;$ENDCMD"&
;;
5) #5 chunk-1.0,multiproc s-zlib4,in
$XT0 "$LXR/zmqImageSource.py --addr=tcp://*:8080 --src rand uint32 256 216 100 -v;$ENDCMD"&
$XT1 "$LXR/zmq2imgGL.py --addr tcp://localhost:8081;$ENDCMD"&
PID0=$!
$XT2 "$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/OutCon5.hdf5 entry/data/dataset --numFrames 100 --timeout 1000 -v --cmpr s-zlib4 --multiproc --clamp in,zmq,addr:tcp://*:8081,qs:4,if:PUSH;$ENDCMD"&
PID1=$!
if [ $NO_UI ]; then
  wait $PID1
  kill $PID0
  wait $PID0
fi
;;
6) #6 chunk-1.0,multiproc s-zlib4,in
SRCHDF=$INDIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/scan_00106.hdf5
#SRCHDF=$INDIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/scan_00106-00161.hdf5
SRCELEM=/entry/data/pilatus_1
ANADIR=$INDIR/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/analysis/data/
DSTELEM=entry/data/out

V=3
#V=63
CLAMP="in,zmq,addr:tcp://localhost:8090,qs:4,if:PUSH in,zmq,addr:tcp://localhost:8091,qs:4,if:PUSH in,zmq,addr:tcp://localhost:8092,qs:4,if:PUSH in,zmq,addr:tcp://localhost:8093,qs:4,if:PUSH"
$XT0 "$LXR/zmqImageSource.py -v --addr=tcp://*:8080 --src hdf $SRCHDF $SRCELEM -s;$ENDCMD"&
$XT1 "$LXR/zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/OutCon60.hdf5 entry/data/dataset --timeout 1000 -v$V --cmpr s-zlib4 --multiproc --clamp $CLAMP;$ENDCMD"&

#$XT2 "$LXR/zmqConsoleSink.py --addr tcp://*:8090 --numFrames 100 -v;$ENDCMD"&
PARAM="--matMsk $ANADIR/pilatus_valid_mask.mat -v$V"
$XT2 "$LXR/procMoment.py --src zmq addr:tcp://*:8090 qs:4 if:PULL timeout:5000 --procmode c --dst hdf $OUTDIR/OutCon61.hdf5 $DSTELEM $PARAM;$ENDCMD"&

#$XT3 "$LXR/zmqConsoleSink.py --addr tcp://*:8091 --numFrames 100 -v;$ENDCMD"&
PARAM="--matRoi $ANADIR/pilatus_valid_mask.mat $ANADIR/pilatus_integration_mask.mat -v$V --cmpr zlib4" #--multiproc has bug?
$XT3 "$LXR/procRoiStat.py --src zmq addr:tcp://*:8091 qs:4 if:PULL timeout:5000 --dst hdf $OUTDIR/OutCon62.hdf5 $DSTELEM $PARAM;$ENDCMD"&

#$XT4 "$LXR/zmqConsoleSink.py --addr tcp://*:8092 --numFrames 100 -v;$ENDCMD"&
PARAM="--roiSize 0 70 -v$V" #--multiproc has bug?
$XT4 "$LXR/procSTXM.py --src zmq addr:tcp://*:8092 queueSz:4 ifType:PULL timeout:5000  $PARAM --dst hdf $OUTDIR/OutCon63.hdf5 $DSTELEM  $PARAM;$ENDCMD"&

#$XT5 "$LXR/zmqConsoleSink.py --addr tcp://*:8093 --numFrames 100 -v;$ENDCMD"&
sed -e s"|\$OUTDIR|$OUTDIR|" -e s"|\$INDIR|$INDIR|" $LXR/test/ptychoZmqOut.json > /tmp/ptycho.json
PARAM="--meta /tmp/ptycho.json --cmpr s-zlib4 -v$V" #--multiproc has bug?
$XT5 "$LXR/procPtycho.py $PARAM;$ENDCMD"&

;;
7) #6 chunk-1.0,multiproc s-zlib4,in
#$XT0 "sleep 10"&
XT 0 'sleep0' 'sleep 10'&
XT 1 'sleep1' 'sleep 3;ls;sleep 2'&
;;
*) # ------- bug ---------
echo "Unknown parameters"
;;
esac
}

for idx in $*; do
  echo RunTest $idx
  RunTest $idx
  WaitAll
done


