#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2015 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
#
#  ./zmqImageSourceTomCatTest.py --addr=tcp://*:8080 --src inc uint32 256 216 100 -v
#
#  OUTDIR=/scratch/tmp/libDetXRTest
# ./zmqWriter.py --addr tcp://localhost:8080 --dst hdf $OUTDIR/zmqWriter32$PF.hdf5 entry/data/dataset --timeout 10000 --numFrames=100 --msgHdlr zmqWriterMsgHandlerTomCat
#
#
# Beispiel:
# {
#    u'tags': [u'0'],
#    u'endianess': u'little',
#    u'frame': 800,
#    u'source':u'gigafrost',
#    u'shape': [1152, 1296],
#    u'htype': [u'array-1.0'],
#    u'type': u'uint16'
#
#    u'image_attributes':
#    {
#     u'exposure_time': 50000,
#     u'scan_id': 0,
#     u'enable': 0,
#     u'sync_time': 2829895658,
#     u'scan_time': 1440156266,
#     u'timestamp': 40432701,
#     u'image_id': 800,
#     u'trigger': 0,
#     u'do_not_store': 0,
#     u'is_complete': True
#    },
# }

import zmq, h5py
import utilities as ut
import numpy as np


class zmqWriterMsgHandlerTomCat:
  '''Message Handler for Tomcat Beamline'''

  def __init__(self,parent):
    self.parent=parent
    self.meta=[]
    self.compound=True

  def Start(self,sock):
    print 'Start',self
    self.sock=sock

  def Process(self,header):
    sock=self.sock
    sock.getsockopt(zmq.RCVMORE)
    content= sock.recv(copy=False)
    self.parent.AddChunk(header,content.bytes)

    try:
      frmNo=header['frame']
      imgAttr=header['image_attributes']
    except KeyError:
      return 1

    #prs=txt.split()
    #prs=map(float,prs)

    try:
      dataL=self.dataL
    except AttributeError:
      k=imgAttr.keys()
      #sorted(range(len(k)), key=lambda kk: k[kk])
      v=imgAttr.values()
      dtype = np.dtype(zip(k, map(type,v)))
      dataL=np.ndarray(shape=(1,),dtype=dtype)
    idx=self.parent.dsObj.ofsIdx+frmNo
    sh=dataL.shape
    if idx+1>sh[0]:
      dataL.resize(idx+1,refcheck=False)
    v=imgAttr.values()
    dataL[idx]=tuple(v)
    self.dataL=dataL
    #try:
    #  self.meta.append((frmNo,imgAttr))
    #except KeyError: pass
    #try: print header['timestamp']
    #except KeyError: pass
    return 1

  def Stop(self):
    print 'Stop',self
    dsName=self.parent.args.dst[2]
    dsName=dsName[dsName.rfind('/')+1:]
    if self.compound:
      gid=ut.CreateGrpPath(self.parent.fid,'measurement/instrument/acquisition')
      grp=h5py.Group(gid)
      try:
        meta=grp.create_dataset(dsName,  data=self.dataL)
      except RuntimeError as e:
        print 'RuntimeError',e
        print 'dsName',dsName
        print 'data',self.dataL
    else:
      gid=ut.CreateGrpPath(self.parent.fid,'measurement/instrument/acquisition/'+dsName)
      grp=h5py.Group(gid)
      dt=self.dataL.dtype
      for k in dt.fields.keys():
        try:
          meta=grp.create_dataset(k,  data=self.dataL[k])
        except RuntimeError as e:
          print 'RuntimeError',e
          print 'dsName',dsName
          print 'k',k
          print 'data',self.dataL[k]





